# -*- coding:utf-8 -*-
# CREATED BY: jiangbohuai
# CREATED ON: 2021/1/25 5:43 PM
# LAST MODIFIED ON:
# AIM: 子句权重list


weights="""
, and 10
, or 10
, but 10
, even 10
, however 10
, whenever 10
, whatever 10
, although 10
, thought 10
, nevertheless 10
, because 10
, besides 10
, so that 10
, otherwise 10
, hence 10
, therefore 10
, meanwhile 10
, where 10
, what 10
, which 10
, when 10
, who 10
, whom 10
, thereby 10
, for 5
, to 5
, I 10
, you 10 
, he 10
, she 10
, there 5
, then 5
, that 5
, following 10
, followed 10
, after 5 
"""