data = '''
Dr.
U!S!A!
No.
abbr.
Brig.
Ltd.,
b.
N.
hr.
Fr.
U.S.A.
Is.
Ms.
F.
N. 
Con.
Sun.
det.
Br.
cont.
cp.
Rep.
Dept.
Thur.
ref.
sq.
Cpl.
f.o.b.
ib.
inc.
par.
cert.
pron.
Co.
R.
Gt.
m.
I.
R.S.V.P.
sing.
ibid.
temp.
Maj.
Sep.
Tues.
Hon.
Pl.
anon.
Corp.
o.n.o.
W.
adj.
Dem.
Mar.
A.M.
Jun.
Rev.
O.
o.b.o.
adv.
Co.,
LL.D
Soc.
Aug.
Sat.
C.E.
So.
Y.
M.A.
exclam.
mm.
gm.
T.
enc.
G.
H.
B.Phil
C.
Jan.
Sec.
max.
U.S.
B.A.
Ct.
p.m.
tel.
n.
st.
N.Y.
prep.
M.S.
Cdr.
p.p.
lat.
Q.
B.S.
esp.
P.M.
Ind.
U. S.
Wed.
chap.
l.c.
doz.
R.I.P.
pl.
Secy.
A.D.
ft.
eng.
p.d.q.
ct.
Sq.
Capt.
q.v.
col.
D.O.B.
pop.
Lt.
Bros.
sec.
A.H.
L.
Av.
Lieut.
M.
u.c.
Mr.
conj.
Col.
r.h.
Mgr.
Inc.
incl.
yr.
P.
Washingto, D.C.
Assoc.
Oct.
neg.
S. 
ext.
Sgt.
govt.
encl.
H.E.
X.
M.O.
arr.
pt.
p.
Apr.
G.L.O.W.
a.m.
l.
H.M.
l.h.
f.
d.
O.R.P.
deg.
A.
Messrs.
H.K.
B.Litt
vol.
dd.
lb.
Asst.
c.
Esq.
Mon.
med.
Ft.
D.
D.C.
D.A.
Mt.
B.
Ave.
N.Z.
V.
Q.E.D.
S.E.
N.B.
Mrs.
B.C.E.
Univ.
lng.
p.w.
Blvd.
Jul.
min.
i.e.
E.
B.C.
C.-in-C.
U.
fig.
h.p.
Sr.
J.
e.g.
D.T.'s
gal.
Supt.
para.
B.Ed.
symb.
T.M.
Prof.
K.
LL.B
U.K.
P.E.
Op.
Gen.
L.A.
dep.
ff.
Ph.D.
D.Litt
do.
ed.
p.s.i.
Feb.
Sept.
viz.
Fri.
Z.
p.a.
Div.
cf.
S.
St.
cu.
attn.
Jr.
Lab.
Dec.
Rd.
d.b.a.
U.N.
Nov.
Sen.
cent.
LL.M
B.Mus
'''
